"use strict";
/**
 * @module gArray
 * @type function tools
 * @description 针对数组的一些工具函数
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.sample = exports.shuffe = exports.indexOfAll = exports.countByValue = exports.countBy = exports.compact = exports.chunk = exports.arrayLike2Array = exports.getTreeChains = exports.array2Object = exports.tree2Array = exports.array2Tree = exports.differenceAll = exports.difference = exports.union = exports.intersectionAll = exports.intersection = exports.unique = exports.deepFlatten = exports.isArrayLike = exports.forEachRight = exports.forEach = exports.forEachType = void 0;
const gObj_1 = require("./gObj");
const gType_1 = require("./gType");
/**
 * @title forEachType
 * @description 从左/右（left/right）遍历数组 柯里化函数
 * @param type "left" | "right"
 * @param arr T[]
 * @param cb (value: T, index: number, array: T[]) => void
 * @returns array T[]
 * @example forEachType("right")([1,2,3],(v,k,arr)=>console.log(v,k,arr)]);
 */
const forEachType = (type = "left") => (arr, cb) => {
    const list = type === "left" ? arr : [...arr].reverse();
    list.forEach(cb);
};
exports.forEachType = forEachType;
/**
 * @title forEach
 * @description 左遍历
 * @param arr T[]
 * @param cb (value: T, index: number, array: T[]) => void
 * @returns array T[]
 */
const forEach = forEachType();
exports.forEach = forEach;
/**
 * @title forEachRight
 * @description 右遍历
 * @param arr T[]
 * @param cb (value: T, index: number, array: T[]) => void
 * @returns array T[]
 */
const forEachRight = forEachType("right");
exports.forEachRight = forEachRight;
/**
 * @title isArrayLike
 * @description 判断是否类数组
 * @param val any
 * @returns boolean
 */
const isArrayLike = (val) => {
    try {
        return "length" in val;
    }
    catch {
        return false;
    }
};
exports.isArrayLike = isArrayLike;
/**
 * @title flatten
 * @description 拉平数组
 * @param arr array
 * @param depth depth 最多拉平的层级 默认1
 * @returns array
 */
const flatten = (arr, depth = 1) => arr.reduce((a, v) => a.concat(depth > 1 && Array.isArray(v) ? flatten(v, depth - 1) : v), []);
/**
 * @title deepFlatten
 * @description 深度拉平数组
 * @param arr array
 * @returns array
 */
const deepFlatten = (arr) => [].concat(...arr.map((v) => (Array.isArray(v) ? deepFlatten(v) : v)));
exports.deepFlatten = deepFlatten;
/**
 * @title unique
 * @description 数组去重
 * @param arr array
 * @returns array
 */
const unique = (arr) => [...new Set(arr)];
exports.unique = unique;
/**
 * @title intersection
 * @description 获取数组交集
 * @param arrT []数组1
 * @param arrR []数组2
 * @returns []两数组同栈集合
 */
const intersection = (arrT, arrR) => {
    return arrT.filter((m) => arrR.includes(m));
};
exports.intersection = intersection;
/**
 * @title intersectionAll
 * @description 取多个数组的交集
 * @param ...arr [][]多个数组
 * @returns 数组交集
 */
const intersectionAll = (...args) => args.reduce((acc, val) => intersection(val, acc));
exports.intersectionAll = intersectionAll;
/**
 * @title union
 * @description 多个元素的并集(去重)
 * @param ...arr [][]多个数组
 * @returns 数组并集
 */
const union = (...args) => unique(flatten(args, 1));
exports.union = union;
/**
 * @title difference
 * @description 两个数组的差集
 * @param arrT []数组1
 * @param arrR []数组2
 * @returns 数组差集
 */
const difference = (arrT, arrR) => {
    const allList = union(arrT, arrR);
    const intersectionList = intersection(arrT, arrR);
    return allList.filter((item) => !intersectionList.includes(item));
};
exports.difference = difference;
/**
 * @title differenceAll
 * @description 多个数组的差集
 * @param ...arr [][]多个数组
 * @returns 数组差集
 */
const differenceAll = (...args) => args.reduce((acc, val) => difference(val, acc));
exports.differenceAll = differenceAll;
/**
 * @title array2Tree
 * @description 将单层级数组转化为树形结构
 * @description parentId为父元素的唯一标识，id为元素的唯一标识，默认为'id', pid为元素的父元素标识，默认为'pid'，children为要生成多层级子元素的字段名，默认为'children'
 * @param arr 扁平数组
 * @param parentId
 * @param options <{ id = "id", pid = "pid", children = "children" }>
 * @returns 树形数组
 */
const array2Tree = (arr, parentId = 0, { id = "id", pid = "pid", children = "children" } = {}) => arr
    .filter((item) => item[pid] === parentId)
    .map((item) => ({
    ...item,
    [children]: array2Tree(arr, item[id], { id, pid, children }),
}));
exports.array2Tree = array2Tree;
/**
 * @title tree2Array
 * @description 树状结构转为一维数组
 * @description id为每个元素的唯一标识，默认为'id'，children为多层级的子元素列表字段，默认为'children'
 * @param arr 扁平数组
 * @param parentId
 * @param options <{ id = "id", children = "children" }>
 * @returns 一维数组
 */
const tree2Array = (tree, { id = "id", children = "children" } = {}) => {
    const list = tree.reduce((acc, item) => {
        if (Array.isArray(item[children]) && item[children].length) {
            return [...acc, item, ...tree2Array(item[children], { id, children })];
        }
        return [...acc, item];
    }, []);
    return list.map((item) => {
        return gObj_1.removeKeys(item, [children]);
    });
};
exports.tree2Array = tree2Array;
/**
 * @title array2Object
 * @description 数组转为对象
 * @description 如果数组元素为对象时指定对象的某个唯一字段为 key 值，没有指定则默认为下标索引值
 * @param arr 数组
 * @param key
 * @returns object
 */
const array2Object = (arr, key) => {
    if (!arr.length) {
        console.warn("传入数组为空");
        return null;
    }
    return arr.reduce((acc, val, index) => {
        acc[key && val[key] ? val[key] : index] = val;
        return acc;
    }, {});
};
exports.array2Object = array2Object;
/**
 * @title getTreeChains
 * @description  根据标识获取树状结构的数据链
 * @param id  number|string
 * @param tree  <{children:[]}>[]
 * @param options  <{ id: "id", pId: "pId", children: "children" }>
 * @returns array
 */
const getTreeChains = (id, tree, filter = ["id"], options = {}) => {
    let opts = {
        id: "id",
        pId: "pId",
        children: "children",
    };
    opts = { ...opts, ...options };
    let list = tree2Array(tree, {
        id: opts.id,
        children: opts.children,
    }).reverse();
    let currentId = id;
    list = list.reduce((acc, item) => {
        if (currentId === item[opts.id]) {
            const chainItem = filter.reduce((subject, key) => {
                subject[key] = item[key];
                return subject;
            }, {});
            acc.unshift(chainItem);
            currentId = item[opts.pId];
        }
        return acc;
    }, []);
    if (filter.length === 1) {
        list = list.map((item) => item[filter[0]]);
    }
    return list;
};
exports.getTreeChains = getTreeChains;
/**
 * @title arrayLike2Array
 * @description 类数组转为数组
 * @param obj Iterable<T> | ArrayLike<T>
 * @returns T[]
 */
const arrayLike2Array = (obj) => {
    if (!isArrayLike(obj)) {
        console.warn("当前传入数据不是类数组");
        return [];
    }
    return Array.from(obj);
};
exports.arrayLike2Array = arrayLike2Array;
/**
 * @title chunk
 * @description 根据给定长度进行分组
 * @param arr T[]
 * @param size number
 * @returns T[][]
 */
const chunk = (arr, size) => Array.from({ length: Math.ceil(arr.length / size) }, (item, index) => arr.slice(size * index, size * (index + 1)));
exports.chunk = chunk;
/**
 * @title compact
 * @description 过滤列表中的空数据
 * @param arr T[]
 * @returns T[]
 */
const compact = (arr) => arr.filter(Boolean);
exports.compact = compact;
/**
 * @title countBy
 * @description 根据条件获取元素的出现次数
 * @param arr T[]
 * @returns T[]
 */
const countBy = (arr, fn = (item) => item) => arr
    .map((item) => (gType_1.isFunction(fn) ? fn(item) : item))
    .reduce((acc, item) => {
    acc[item] = acc[item] ? ++acc[item] : 1;
    return acc;
}, {});
exports.countBy = countBy;
/**
 * @title countByValue
 * @description 获取指定元素的出现次数
 * @param arr
 * @param val
 * @returns number
 */
const countByValue = (arr, val) => countBy(arr)[val];
exports.countByValue = countByValue;
/**
 * @title indexOfAll
 * @description 获取指定元素的下标值
 * @param arr
 * @param val
 * @returns number[]
 */
const indexOfAll = (arr, val) => arr.reduce((acc, item, index) => (item === val ? [...acc, index] : acc), []);
exports.indexOfAll = indexOfAll;
/**
 * @title shuffe
 * @description 随机排序
 * @param arr
 * @returns number[]
 */
const shuffe = (arr) => {
    const list = [...arr];
    let len = list.length;
    while (len) {
        const i = Math.floor(Math.random() * len--);
        [list[len], list[i]] = [list[i], list[len]];
    }
    return list;
};
exports.shuffe = shuffe;
/**
 * @title sample
 * @description 随机取数组中数据
 * @description size为取出元素的个数
 * @param arr any[]
 * @param size number
 * @returns any[]
 * @example sample([1, 2, 3, 4, 5, 6, 7, 8, 9], 3) 结果=>[4, 6, 8]
 */
const sample = (arr, size = 1) => {
    const list = shuffe(arr);
    return list.slice(0, size);
};
exports.sample = sample;
//# sourceMappingURL=gArray.js.map